==> sisu_4.2.3.orig.tar.xz: sisu-4.2.3/lib/sisu/v5/xml_fictionbook.rb <==
# encoding: utf-8
=begin

 * Name: SiSU

 * Description: a framework for document structuring, publishing and search

 * Author: Ralph Amissah

 * Copyright: (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
   2007, 2008, 2009, 2010, 2011, 2012, 2013 Ralph Amissah, All Rights Reserved.

 * License: GPL 3 or later:

   SiSU, a framework for document structuring, publishing and search

   Copyright (C) Ralph Amissah

   This program is free software: you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation, either version 3 of the License, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along with
   this program. If not, see <http://www.gnu.org/licenses/>.

   If you have Internet connection, the latest version of the GPL should be
   available at these locations:
   <http://www.fsf.org/licensing/licenses/gpl.html>
   <http://www.gnu.org/licenses/gpl.html>

   <http://www.sisudoc.org/sisu/en/manifest/gpl.fsf.html>

 * SiSU uses:
   * Standard SiSU markup syntax,
   * Standard SiSU meta-markup syntax, and the
   * Standard SiSU object citation numbering and system

 * Hompages:
   <http://www.jus.uio.no/sisu>
   <http://www.sisudoc.org>

 * Download:
   <http://www.sisudoc.org/sisu/en/SiSU/download.html>

 * Git
   <http://sources.sisudoc.org/gitweb/?p=code/sisu.git;a=summary>
   <http://sources.sisudoc.org/?p=code/sisu.git;a=blob;f=lib/sisu/v5/xml_fictionbook.rb;hb=HEAD>

 * Ralph Amissah
   <ralph@amissah.com>
   <ralph.amissah@gmail.com>

 ** Description: extract and print an XML rendition of document structure to screen

=end
module SiSU_XML_Fictionbook
  require_relative 'dal'                                # dal.rb
  require_relative 'sysenv'                             # sysenv.rb
    include SiSU_Env
  require_relative 'shared_txt'                         # shared_txt.rb
    include SiSU_TextUtils
  require_relative 'shared_xml'                         # shared_xml.rb
    include SiSU_XML_Munge
  include SiSU_Param
  class Source
    def initialize(opt)
      @opt=opt
      @sp='  '
    end
    def read
      begin
        @md=SiSU_Param::Parameters.new(@opt).get
        @dal_array=SiSU_DAL::Source.new(@opt).get
        SiSU_XML_Fictionbook::Source::Scroll.new(@dal_array,@md).songsheet
      rescue
        SiSU_Errors::Rescued.new($!,$@,@opt.cmd,@opt.fns).location do
          __LINE__.to_s + ':' + __FILE__
        end
      ensure
      end
    end
    private
    class Scroll <Source
      def initialize(data='',md='')
        @data,@md=data,md
        @trans=SiSU_XML_Munge::Trans.new(@md)
        @file_fictionbook=SiSU_Env::FileOp.new(@md,'fictionbook.xml').mkfile
      end
      def songsheet
        @t='fictionbook'
        @s=['section',          #@s=['body',
          'section',
          'section',
          'section',
          'section',
          'section',
          'section'
        ]
        head
        extract_endnotes
        structure
        tail
      end
      def head
        version=SiSU_Env::InfoVersion.instance.get_version
        rb_ver=SiSU_Env::InfoVersion.instance.rbversion
        date_available=if defined? @md.date.available; "\n     <p>#{@md.date.available} Initial version</p>"
        else ''
        end
        date_modified=if defined? @md.date.modified; "\n      <p>#{@md.date.modified} Last Modified</p>"
        else ''
        end
        head=<<WOK
<?xml version="1.0" encoding="UTF-8"?>
<FictionBook xmlns:xlink="http://www.w3.org/1999/xlink"
 xmlns="http://www.gribuser.ru/xml/fictionbook/2.0">
<description>
  <title-info>
    <genre match="100">***</genre>
    <author>
      <first-name>***</first-name>
      <middle-name>***</middle-name>
      <last-name>***</last-name>
    </author>
    <book-title>#{@md.title.full}</book-title>
    <annotation>
    </annotation>
    <date value="#{@md.date}">yyyy</date>
  </title-info>
     <document-info>
    <author>
      <first-name/>
      <last-name/>
      <nickname/>
    </author>
    <program-used>#{version[:project]} #{version[:version]} and #{rb_ver}</program-used>
    <date value="#{version[:date]}">#{version[:date]}</date>
    <src-ocr/>
    <version>1.0</version>
    <history>#{date_available}#{date_modified}
    </history>
  </document-info>
</description>
<body>
WOK
        put(head)
      end
      def extract_endnotes
        @endnotes=[]
        @data.each do |para|
          @endnotes << para.scan(/~\{(.+?)\}~/m)
        end
        @endnotes=@endnotes.flatten
      end
      def endnotes
        @endnotes.each do |endnote|
          endnote=endnote.strip
          endnote=@trans.markup_fictionbook(endnote)
          endnote="<p>#{endnote}</p>"
          util=SiSU_TextUtils::Wrap.new(endnote,80,10)
          endnote=util.line_wrap
          put(endnote)
        end
      end
      def tail
        tail=<<WOK
</body>
</FictionBook>
WOK
        put(tail)
      end
      def markup(para,type='')
        para=para.strip
        para=@trans.markup_fictionbook(para)
        para=if type.empty?; "<p>#{para}</p>"
        else "<#{type}><p>#{para}</p></#{type}>"
        end
        util=SiSU_TextUtils::Wrap.new(para,80,10)
        util.line_wrap
      end
      def put(line)
        @file_fictionbook.puts line
        puts line if @md.opt.cmd =~/V/
      end
      def structure_build_tag_close(lev,h)
        @sp='  '
        case h[0]
        when 1
          put("#{@sp*1}</#{@s[1]}>") if (lev <= 1) and h[1]
          put("</#{@s[0]}>")         if (lev==0)
        when 2
          put("#{@sp*2}</#{@s[2]}>") if (lev <= 2) and h[2]
          put("#{@sp*1}</#{@s[1]}>") if (lev <= 1) and h[1]
          put("</#{@s[0]}>")         if (lev==0)
        when 3
          put("#{@sp*3}</#{@s[3]}>") if (lev <= 3) and h[3]
          put("#{@sp*2}</#{@s[2]}>") if (lev <= 2) and h[2]
          put("#{@sp*1}</#{@s[1]}>") if (lev <= 1) and h[1]
          put("</#{@s[0]}>")         if (lev==0)
        when 4
          put("#{@sp*4}</#{@s[4]}>") if (lev <= 4)
          put("#{@sp*3}</#{@s[3]}>") if (lev <= 3) and h[3]
          put("#{@sp*2}</#{@s[2]}>") if (lev <= 2) and h[2]
          put("#{@sp*1}</#{@s[1]}>") if (lev <= 1) and h[1]
          put("</#{@s[0]}>")         if (lev==0)
        when 5
          put("#{@sp*5}</#{@s[5]}>") if (lev <= 5)
          put("#{@sp*4}</#{@s[4]}>") if (lev <= 4)
          put("#{@sp*3}</#{@s[3]}>") if (lev <= 3) and h[3]
          put("#{@sp*2}</#{@s[2]}>") if (lev <= 2) and h[2]
          put("#{@sp*1}</#{@s[1]}>") if (lev <= 1) and h[1]
          put("</#{@s[0]}>")         if (lev==0)
        when 6
          put("#{@sp*6}</#{@s[6]}>") if (lev <= 6)
          put("#{@sp*5}</#{@s[5]}>") if (lev <= 5)
          put("#{@sp*4}</#{@s[4]}>") if (lev <= 4)
          put("#{@sp*3}</#{@s[3]}>") if (lev <= 3) and h[3]
          put("#{@sp*2}</#{@s[2]}>") if (lev <= 2) and h[2]
          put("#{@sp*1}</#{@s[1]}>") if (lev <= 1) and h[1]
          put("</#{@s[0]}>")         if (lev==0)
        end
      end
      def structure_build(ds)
        @h=[0,false,false,false]
        put("<#{@s[0]}>")
        ds.each_with_index do |x,i|
          @ef=false
          case x[:lev]
          when /^1/
            structure_build_tag_close(1,@h)
            y="#{@sp*1}<#{@s[1]}>\n#{x[:para]}"
            @h=[1,true,false,false]
          when /^2/
            structure_build_tag_close(2,@h)
            y="#{@sp*2}<#{@s[2]}>\n#{x[:para]}"
            @h=[2,true,true,false]
          when /^0:0/ #endnotes and metadata
            structure_build_tag_close(2,@h)
            y="#{@sp*2}<#{@s[2]}>\n#{x[:para]}"
            @h=[2,true,true,false]
            @ef=true if x[:hdr] =~/u0/
          when /^3/
            structure_build_tag_close(3,@h)
            y="#{@sp*3}<#{@s[3]}>\n#{x[:para]}"
            @h=[3,true,true,true]
          when /^4/
            structure_build_tag_close(4,@h)
            y="#{@sp*4}<#{@s[4]}>\n#{x[:para]}"
            @h[0]=4
          when /^m2/ #metadata
            structure_build_tag_close(4,@h)
            y="#{@sp*4}<#{@s[4]}>\n#{x[:para]}"
            @h[0]=4
          when /^5/
            structure_build_tag_close(5,@h)
            y="#{@sp*5}<#{@s[5]}>\n#{x[:para]}"
            @h[0]=5
          when /^6/
            structure_build_tag_close(6,@h)
            y="#{@sp*6}<#{@s[6]}>\n#{x[:para]}"
            @h[0]=6
          else
            y=if @md.opt.cmd =~/V/; "#{x[:para]}"
            else nil
            end
          end
          put(y) if y
          endnotes if @ef
        end
        structure_build_tag_close(0,@h)
      end
      def structure
        data=@data
        @ds=[]
        c=0
        data.each do |para|
          rgx_headers=/#{Mx[:id_o]}~(\d+);((?:\w|[0-6]:)\d+);(\w\d+)#{Mx[:id_c]}/ #fix
          if para =~rgx_headers
            x=(rgx_headers).match(para)
            if x[3] =~/^[hum]\d+/
              @ds[c]={}
              @ds[c][:ocn]=x[1]
              @ds[c][:lev]=x[2]
              @ds[c][:hdr]=x[3]
              @ds[c][:para]=markup(para,'title')
            else
              @ds[c]={}
              @ds[c][:para]=markup(para) if @md.opt.cmd =~/V/
            end
            c+=1
          end
        end
        structure_build(@ds)
        @ds
      end
    end
  end
end
__END__

==> sisu_4.2.8.orig.tar.xz: sisu-4.2.8/lib/sisu/v5/xml_fictionbook.rb <==
# encoding: utf-8
=begin

 * Name: SiSU

 * Description: a framework for document structuring, publishing and search

 * Author: Ralph Amissah

 * Copyright: (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
   2007, 2008, 2009, 2010, 2011, 2012, 2013 Ralph Amissah, All Rights Reserved.

 * License: GPL 3 or later:

   SiSU, a framework for document structuring, publishing and search

   Copyright (C) Ralph Amissah

   This program is free software: you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation, either version 3 of the License, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along with
   this program. If not, see <http://www.gnu.org/licenses/>.

   If you have Internet connection, the latest version of the GPL should be
   available at these locations:
   <http://www.fsf.org/licensing/licenses/gpl.html>
   <http://www.gnu.org/licenses/gpl.html>

   <http://www.sisudoc.org/sisu/en/manifest/gpl.fsf.html>

 * SiSU uses:
   * Standard SiSU markup syntax,
   * Standard SiSU meta-markup syntax, and the
   * Standard SiSU object citation numbering and system

 * Hompages:
   <http://www.jus.uio.no/sisu>
   <http://www.sisudoc.org>

 * Download:
   <http://www.sisudoc.org/sisu/en/SiSU/download.html>

 * Git
   <http://sources.sisudoc.org/gitweb/?p=code/sisu.git;a=summary>
   <http://sources.sisudoc.org/?p=code/sisu.git;a=blob;f=lib/sisu/v5/xml_fictionbook.rb;hb=HEAD>

 * Ralph Amissah
   <ralph@amissah.com>
   <ralph.amissah@gmail.com>

 ** Description: extract and print an XML rendition of document structure to screen

=end
module SiSU_XML_Fictionbook
  require_relative 'dal'                                # dal.rb
  require_relative 'sysenv'                             # sysenv.rb
    include SiSU_Env
  require_relative 'shared_txt'                         # shared_txt.rb
    include SiSU_TextUtils
  require_relative 'shared_xml'                         # shared_xml.rb
    include SiSU_XML_Munge
  include SiSU_Param
  class Source
    def initialize(opt)
      @opt=opt
      @sp='  '
    end
    def read
      begin
        @md=SiSU_Param::Parameters.new(@opt).get
        @dal_array=SiSU_DAL::Source.new(@opt).get
        SiSU_XML_Fictionbook::Source::Scroll.new(@dal_array,@md).songsheet
      rescue
        SiSU_Errors::Rescued.new($!,$@,@opt.cmd,@opt.fns).location do
          __LINE__.to_s + ':' + __FILE__
        end
      ensure
      end
    end
    private
    class Scroll <Source
      def initialize(data='',md='')
        @data,@md=data,md
        @trans=SiSU_XML_Munge::Trans.new(@md)
        @file_fictionbook=SiSU_Env::FileOp.new(@md,'fictionbook.xml').mkfile
      end
      def songsheet
        @t='fictionbook'
        @s=['section',          #@s=['body',
          'section',
          'section',
          'section',
          'section',
          'section',
          'section'
        ]
        head
        extract_endnotes
        structure
        tail
      end
      def head
        version=SiSU_Env::InfoVersion.instance.get_version
        rb_ver=SiSU_Env::InfoVersion.instance.rbversion
        date_available=if defined? @md.date.available; "\n     <p>#{@md.date.available} Initial version</p>"
        else ''
        end
        date_modified=if defined? @md.date.modified; "\n      <p>#{@md.date.modified} Last Modified</p>"
        else ''
        end
        head=<<WOK
<?xml version="1.0" encoding="UTF-8"?>
<FictionBook xmlns:xlink="http://www.w3.org/1999/xlink"
 xmlns="http://www.gribuser.ru/xml/fictionbook/2.0">
<description>
  <title-info>
    <genre match="100">***</genre>
    <author>
      <first-name>***</first-name>
      <middle-name>***</middle-name>
      <last-name>***</last-name>
    </author>
    <book-title>#{@md.title.full}</book-title>
    <annotation>
    </annotation>
    <date value="#{@md.date}">yyyy</date>
  </title-info>
     <document-info>
    <author>
      <first-name/>
      <last-name/>
      <nickname/>
    </author>
    <program-used>#{version[:project]} #{version[:version]} and #{rb_ver}</program-used>
    <date value="#{version[:date]}">#{version[:date]}</date>
    <src-ocr/>
    <version>1.0</version>
    <history>#{date_available}#{date_modified}
    </history>
  </document-info>
</description>
<body>
WOK
        put(head)
      end
      def extract_endnotes
        @endnotes=[]
        @data.each do |para|
          @endnotes << para.scan(/~\{(.+?)\}~/m)
        end
        @endnotes=@endnotes.flatten
      end
      def endnotes
        @endnotes.each do |endnote|
          endnote=endnote.strip
          endnote=@trans.markup_fictionbook(endnote)
          endnote="<p>#{endnote}</p>"
          util=SiSU_TextUtils::Wrap.new(endnote,80,10)
          endnote=util.line_wrap
          put(endnote)
        end
      end
      def tail
        tail=<<WOK
</body>
</FictionBook>
WOK
        put(tail)
      end
      def markup(para,type='')
        para=para.strip
        para=@trans.markup_fictionbook(para)
        para=if type.empty?; "<p>#{para}</p>"
        else "<#{type}><p>#{para}</p></#{type}>"
        end
        util=SiSU_TextUtils::Wrap.new(para,80,10)
        util.line_wrap
      end
      def put(line)
        @file_fictionbook.puts line
        puts line if @md.opt.act[:verbose_plus][:set]==:on
      end
      def structure_build_tag_close(lev,h)
        @sp='  '
        case h[0]
        when 1
          put("#{@sp*1}</#{@s[1]}>") if (lev <= 1) and h[1]
          put("</#{@s[0]}>")         if (lev==0)
        when 2
          put("#{@sp*2}</#{@s[2]}>") if (lev <= 2) and h[2]
          put("#{@sp*1}</#{@s[1]}>") if (lev <= 1) and h[1]
          put("</#{@s[0]}>")         if (lev==0)
        when 3
          put("#{@sp*3}</#{@s[3]}>") if (lev <= 3) and h[3]
          put("#{@sp*2}</#{@s[2]}>") if (lev <= 2) and h[2]
          put("#{@sp*1}</#{@s[1]}>") if (lev <= 1) and h[1]
          put("</#{@s[0]}>")         if (lev==0)
        when 4
          put("#{@sp*4}</#{@s[4]}>") if (lev <= 4)
          put("#{@sp*3}</#{@s[3]}>") if (lev <= 3) and h[3]
          put("#{@sp*2}</#{@s[2]}>") if (lev <= 2) and h[2]
          put("#{@sp*1}</#{@s[1]}>") if (lev <= 1) and h[1]
          put("</#{@s[0]}>")         if (lev==0)
        when 5
          put("#{@sp*5}</#{@s[5]}>") if (lev <= 5)
          put("#{@sp*4}</#{@s[4]}>") if (lev <= 4)
          put("#{@sp*3}</#{@s[3]}>") if (lev <= 3) and h[3]
          put("#{@sp*2}</#{@s[2]}>") if (lev <= 2) and h[2]
          put("#{@sp*1}</#{@s[1]}>") if (lev <= 1) and h[1]
          put("</#{@s[0]}>")         if (lev==0)
        when 6
          put("#{@sp*6}</#{@s[6]}>") if (lev <= 6)
          put("#{@sp*5}</#{@s[5]}>") if (lev <= 5)
          put("#{@sp*4}</#{@s[4]}>") if (lev <= 4)
          put("#{@sp*3}</#{@s[3]}>") if (lev <= 3) and h[3]
          put("#{@sp*2}</#{@s[2]}>") if (lev <= 2) and h[2]
          put("#{@sp*1}</#{@s[1]}>") if (lev <= 1) and h[1]
          put("</#{@s[0]}>")         if (lev==0)
        end
      end
      def structure_build(ds)
        @h=[0,false,false,false]
        put("<#{@s[0]}>")
        ds.each_with_index do |x,i|
          @ef=false
          case x[:lev]
          when /^1/
            structure_build_tag_close(1,@h)
            y="#{@sp*1}<#{@s[1]}>\n#{x[:para]}"
            @h=[1,true,false,false]
          when /^2/
            structure_build_tag_close(2,@h)
            y="#{@sp*2}<#{@s[2]}>\n#{x[:para]}"
            @h=[2,true,true,false]
          when /^0:0/ #endnotes and metadata
            structure_build_tag_close(2,@h)
            y="#{@sp*2}<#{@s[2]}>\n#{x[:para]}"
            @h=[2,true,true,false]
            @ef=true if x[:hdr] =~/u0/
          when /^3/
            structure_build_tag_close(3,@h)
            y="#{@sp*3}<#{@s[3]}>\n#{x[:para]}"
            @h=[3,true,true,true]
          when /^4/
            structure_build_tag_close(4,@h)
            y="#{@sp*4}<#{@s[4]}>\n#{x[:para]}"
            @h[0]=4
          when /^m2/ #metadata
            structure_build_tag_close(4,@h)
            y="#{@sp*4}<#{@s[4]}>\n#{x[:para]}"
            @h[0]=4
          when /^5/
            structure_build_tag_close(5,@h)
            y="#{@sp*5}<#{@s[5]}>\n#{x[:para]}"
            @h[0]=5
          when /^6/
            structure_build_tag_close(6,@h)
            y="#{@sp*6}<#{@s[6]}>\n#{x[:para]}"
            @h[0]=6
          else
            y=if @md.opt.act[:verbose_plus][:set]==:on; "#{x[:para]}"
            else nil
            end
          end
          put(y) if y
          endnotes if @ef
        end
        structure_build_tag_close(0,@h)
      end
      def structure
        data=@data
        @ds=[]
        c=0
        data.each do |para|
          rgx_headers=/#{Mx[:id_o]}~(\d+);((?:\w|[0-6]:)\d+);(\w\d+)#{Mx[:id_c]}/ #fix
          if para =~rgx_headers
            x=(rgx_headers).match(para)
            if x[3] =~/^[hum]\d+/
              @ds[c]={}
              @ds[c][:ocn]=x[1]
              @ds[c][:lev]=x[2]
              @ds[c][:hdr]=x[3]
              @ds[c][:para]=markup(para,'title')
            else
              @ds[c]={}
              @ds[c][:para]=markup(para) if @md.opt.act[:verbose_plus][:set]==:on
            end
            c+=1
          end
        end
        structure_build(@ds)
        @ds
      end
    end
  end
end
__END__
